#import "dock.h"

static GDock *sharedgdock = nil;

@implementation GDock

+ (GDock *)sharedgdock
{
	if(!sharedgdock) {
      NS_DURING
      {
	 	 	[gnustep_global_lock lock];
	  		sharedgdock = [[self alloc] init];
	  		[gnustep_global_lock unlock];
		}
		NS_HANDLER
		{
	  		[gnustep_global_lock unlock];
	  		[localException raise];
		}
      NS_ENDHANDLER
	}
	return sharedgdock;
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	NSRect winRect;
	int xw, yw, ww, hw;
	
	defaults = [NSUserDefaults standardUserDefaults];
	appsPaths = [[defaults arrayForKey: @"appspaths"] retain];	
	winGeom = [[defaults arrayForKey: @"wingeom"] retain];
	if(!winGeom) {
		winGeom = [[NSMutableArray alloc] initWithCapacity: 4];
		[winGeom addObject: [NSNumber numberWithInt: 0]];
		[winGeom addObject: [NSNumber numberWithInt: 0]];
		[winGeom addObject: [NSNumber numberWithInt: 64]];
		[winGeom addObject: [NSNumber numberWithInt: 65]];
		[defaults setObject: winGeom forKey: @"wingeom"];
		[defaults synchronize];
	}
	xw = [[winGeom objectAtIndex: 0] intValue];
	yw = [[winGeom objectAtIndex: 1] intValue];
	ww = [[winGeom objectAtIndex: 2] intValue];
	hw = [[winGeom objectAtIndex: 3] intValue];
	winRect = NSMakeRect(xw, yw, ww, hw);
	
	win = [[GDWindow alloc] initWithRect: winRect];
	[win display];
	[win orderFront:nil];
	[win addApps: appsPaths];
}

- (void)dealloc
{
	[appsPaths release];
	[win release];
	[super dealloc];
}

- (BOOL)applicationShouldTerminate:(NSApplication *)app 
{
	[win calculateFrames];
	return YES;
}

- (void)registerAppsListInDefaults:(NSArray *)appList
{
	[appsPaths release];
	appsPaths = [[NSArray alloc] initWithArray: appList];
	[defaults setObject: appsPaths forKey: @"appspaths"];
	[defaults synchronize];
}

- (void)registerDockGeometryInDefaults:(NSArray *)values
{
	[winGeom release];
	winGeom = [[NSArray alloc] initWithArray: values];
	[defaults setObject: winGeom forKey: @"wingeom"];
	[defaults synchronize];
}

- (void)method:menuCell
{

}

@end
