#include "gdockedapp.h"
#import "dock.h"

@implementation GDockedApp

- (id)initWithAppPath:(NSString *)aPath inDockWin:(GDWindow *)aWin
{
	self = [super initWithFrame: NSMakeRect(0, 0, 64, 64)];
	if(self) {
		dockWin = [aWin retain];
		appPath = [aPath retain];
		tile = [[NSImage imageNamed:@"common_Tile.tiff"] retain];
		appIcon = [[[NSWorkspace sharedWorkspace] iconForFile: appPath] retain];		
		isDragTarget = NO;
		dndok = YES;
  		[self registerForDraggedTypes: [NSArray arrayWithObjects: @"FileObj", @"DockObj", nil]];
	}
	return self;
}

- (void)dealloc
{
	[tile release];
	[appIcon release];
	[appPath release];
	[dockWin release];
	[super dealloc];
}

- (BOOL)acceptsFirstResponder
{
	return YES;
}

- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent
{
	return YES;
}

- (id)delegate
{
  	return delegate;
}

- (void)setDelegate:(id)aDelegate
{
  	[aDelegate retain];
  	[delegate release];
  	delegate = aDelegate;
}

- (NSString *)appPath
{
	return appPath;
}

- (void)setDndOk:(BOOL)value
{
	dndok = value;
}

- (void)mouseDown:(NSEvent*)theEvent
{
	dragdelay = 0;
	dndok = YES;
	if([theEvent clickCount] > 1)
		[[NSWorkspace sharedWorkspace] launchApplication: appPath];
}                                                        

- (void)mouseDragged:(NSEvent *)theEvent
{
	dragdelay++;
	if(dragdelay > 5) 
		[self startExternalDragOnEvent:theEvent];
}

- (void)drawRect:(NSRect)rect
{
	NSPoint imgPoint;
	
	[tile compositeToPoint: NSZeroPoint operation: NSCompositeSourceOver];
  	imgPoint = NSMakePoint((64 - [appIcon size].width)/2, (64 - [appIcon size].height)/2);
	[appIcon compositeToPoint: imgPoint operation: NSCompositeSourceOver];
}

@end

@implementation GDockedApp (DraggingDestination)

- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
{
	NSPasteboard *pb;

	pb = [sender draggingPasteboard];
  	if([[pb types] indexOfObject:@"FileObj"] != NSNotFound
							|| [[pb types] indexOfObject:@"DockObj"] != NSNotFound) {
		isDragTarget = YES;
    	return NSDragOperationAll;
  	}
	return NSDragOperationNone;
}

- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
{
  	return NSDragOperationAll;
}

- (void)draggingExited:(id <NSDraggingInfo>)sender
{
	isDragTarget = NO;
}

- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
{
	return isDragTarget;
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
	return YES;
}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
	NSPasteboard *pb;
	NSData *pbData;
	NSArray *sourcePaths;
	NSString *filePath, *pathExtension;
	BOOL isdir;
	
  	pb = [sender draggingPasteboard];

  	pbData = [pb dataForType: @"FileObj"];
	if(pbData) {
  		sourcePaths = (NSArray *)[NSUnarchiver unarchiveObjectWithData: pbData];
		if([sourcePaths count] > 1) {
			NSRunAlertPanel(nil, @"You can add only one object!", @"Continue", nil, nil);
			isDragTarget = NO;
			return;
		}
		filePath = [sourcePaths objectAtIndex: 0];
		pathExtension = [[filePath pathExtension] lowercaseString];
		if([pathExtension isEqualToString: @"app"]
									|| [pathExtension isEqualToString: @"debug"]) {
			[dockWin addAppWithPath: filePath fromReceiver: self];
		} else {
			[[NSFileManager defaultManager] fileExistsAtPath: filePath isDirectory: &isdir];
			if(!isdir) 
				[[NSWorkspace sharedWorkspace] openFile: filePath withApplication: appPath];
			isDragTarget = NO;
			return;
		}
	} else {
		printf("NO 'FileObj' DND DATA!\n");
	}

  	pbData = [pb dataForType: @"DockObj"];
	if(pbData) {
  		filePath = (NSString *)[NSUnarchiver unarchiveObjectWithData: pbData];
		[dockWin addAppWithPath: filePath fromReceiver: self];
	} else {
		printf("NO 'DockObj' DND DATA!\n");
	}

	isDragTarget = NO;
}

@end

@implementation GDockedApp (DraggingSource)

- (unsigned int)draggingSourceOperationMaskForLocal:(BOOL)flag
{
  	return NSDragOperationAll;
}

- (void)startExternalDragOnEvent:(NSEvent *)event
{
  	NSPasteboard *pb;
  	NSPoint dragPoint, startingPoint;
  	NSSize offset;
	NSEvent *nextEvent;

	pb = [NSPasteboard pasteboardWithName: NSDragPboard];

	nextEvent = [[self window] nextEventMatchingMask:
    							NSLeftMouseUpMask | NSLeftMouseDraggedMask];

  	if([nextEvent type] != NSLeftMouseDragged) 
   	return;

  	startingPoint = [event locationInWindow];
  	startingPoint = [self convertPoint: startingPoint fromView: nil];
  	dragPoint = [nextEvent locationInWindow];
  	dragPoint = [self convertPoint: dragPoint fromView: nil];
  	offset.width = dragPoint.x - startingPoint.x;
  	offset.height = dragPoint.y - startingPoint.y;

  	[self declareAndSetShapeOnPasteboard: pb];
	dndok = NO;
  	if(DelegateHandles(control:shouldStartExternalDragOnEvent:)){
    	if(![delegate control: self shouldStartExternalDragOnEvent: event])
      	return;
  	}

  	[self dragImage: appIcon
                at: [self frame].origin
            offset: offset
             event: event
        pasteboard: pb
            source: self
         slideBack: YES];

  	if(DelegateHandles(control:shouldStartExternalDragOnEvent:)){
    	[delegate control: self didFinishExternalDragOnEvent: event];
  	}

  	[self setNeedsDisplay: YES];
	
	if(!dndok)
		[dockWin removeDockedApp: self];
}

- (void)declareAndSetShapeOnPasteboard:(NSPasteboard *)pb
{
  	NSData *pbData;

  	[pb declareTypes: [NSArray arrayWithObject: @"DockObj"] owner: self];
  	pbData = [NSArchiver archivedDataWithRootObject: appPath];
  	[pb setData: pbData forType: @"DockObj"];
}

@end
