#include "gdwindow.h"
#import "dock.h"

@implementation GDWindow

- (id)initWithRect:(NSRect)aRect;
{
	self = [super initWithContentRect:aRect
									styleMask:NSBorderlessWindowMask
									  backing:NSBackingStoreRetained
										 defer:NO];
 	return self;
}

- (void)dealloc
{
	[dockedLogo release];
	[dockedApps release];	
	[super dealloc];
}

- (void)addApps:(NSArray *)appsPaths
{
	NSString *appPath;
	GDockedApp *dApp;
	int i;

	dockedLogo = [[GDMainLogo alloc] initWithAppPath: nil inDockWin:self];
	[[self contentView] addSubview: dockedLogo];
	dockedApps = [[NSMutableArray alloc] initWithCapacity: 1];
	if(appsPaths) {
		for(i = 0; i < [appsPaths count]; i++) {
			appPath = [appsPaths objectAtIndex: i];
 			dApp = [[GDockedApp alloc] initWithAppPath: appPath inDockWin:self];
			[[self contentView] addSubview: dApp];		
			[dockedApps addObject: dApp];
			[dApp release];
		}
	}
	[self calculateFrames];
}

- (void)addAppWithPath:(NSString *)appPath fromReceiver:(GDockedApp *)aDockApp
{
	GDockedApp *dApp;
	int i, winHeight = 64;
	
	for(i = 0; i <= [dockedApps count]; i++)
		winHeight += 64;
	if(winHeight > [[NSScreen mainScreen] visibleFrame].size.height) {
		NSRunAlertPanel(nil, @"Sorry, the dock is full!", @"Continue", nil, nil);
		return;
	}
	
	for(i = 0; i < [dockedApps count]; i++) {
		dApp = [dockedApps objectAtIndex: i];
		if([[dApp appPath] isEqualToString: appPath]) {
			[dApp removeFromSuperview];	
			[dockedApps removeObject: dApp];
			break;
		}
	}

  	dApp = [[GDockedApp alloc] initWithAppPath: appPath inDockWin:self];
	[[self contentView] addSubview: dApp];
	if(aDockApp == dockedLogo) {
		[dockedApps insertObject: dApp atIndex: 0];
	} else {
		for(i = 0; i < [dockedApps count]; i++) {
			if(aDockApp == [dockedApps objectAtIndex: i]) {
				[dockedApps insertObject: dApp atIndex: i+1];
				break;
			}
		}
	}
	[dApp release];
	
	[self calculateFrames];
}

- (void)removeDockedApp:(GDockedApp *)aDockApp
{
	[aDockApp removeFromSuperview];	
	[dockedApps removeObject: aDockApp];
	[self calculateFrames];
}

- (void)calculateFrames
{
	NSMutableArray *paths;
	GDockedApp *dApp;
	NSRect myRect, dLogoRect, dAppRect;
	long topY, bottomY, newHeight;
	int i;

	newHeight = 64 * [dockedApps count] + 64;
	topY = frame.origin.y + frame.size.height;
	bottomY = topY - newHeight;
	myRect = NSMakeRect(frame.origin.x, bottomY, 64, newHeight);
	[self setFrame: myRect display: YES];

	dLogoRect = NSMakeRect(0, frame.size.height -64, 64, 64);
	[dockedLogo setFrame: dLogoRect];
	[dockedLogo setNeedsDisplay: YES];
	
	for(i = 0; i < [dockedApps count]; i++) {
		dApp = [dockedApps objectAtIndex: i];
		dAppRect = NSMakeRect(0, frame.size.height - (64 * (i+2)), 64, 64);
		[dApp setFrame: dAppRect];
		[dApp setNeedsDisplay: YES];
	}

	paths = [NSMutableArray array];
	for(i = 0; i < [dockedApps count]; i++) {
		dApp = [dockedApps objectAtIndex: i];
		[paths addObject: [dApp appPath]];
	}
	[[GDock sharedgdock] registerAppsListInDefaults: paths];
	[self registerPosition];
}

- (void)registerPosition
{
	NSMutableArray *geometry = [NSMutableArray arrayWithCapacity: 4];
	[geometry addObject: [NSNumber numberWithInt:(int)frame.origin.x]];
	[geometry addObject: [NSNumber numberWithInt:(int)frame.origin.y]];
	[geometry addObject: [NSNumber numberWithInt:(int)frame.size.width]];
	[geometry addObject: [NSNumber numberWithInt:(int)frame.size.height]];
	[[GDock sharedgdock] registerDockGeometryInDefaults: geometry];
}

@end
